// Copyright 1999-2017. Parallels IP Holdings GmbH. All Rights Reserved.
import { createElement, cloneElement, Component, Children, Content, AuxActions } from "@plesk/ui-library";
import Hint from "./Hint";
import FormRow from "./FormRow";
import SmallTools from "./SmallTools";
import SmallButton from "./SmallButton";
import InplaceEditor from "./InplaceEditor";
import DirectoryBrowserValue from "./DirectoryBrowserValue";
import Dropdown from "./Dropdown";
import PanelsGroup from "./PanelsGroup";
import SidebarPanel from "./SidebarPanel";
import RunScriptPopup from "./RunScriptPopup";
const locale = Jsw.Locale.getSection('controllers.domain.index');

const addTrailingSlash = path => path.replace(/\/?$/, '/');
const splitPath = path => {
    path = path.split('/');
    const file = path.pop();
    return {
        currentDir: path.join('/') || '/',
        file
    };
};

class Application extends Component {
    constructor(props) {
        super(props);
        this.state = {
            savingField: null
        };
    }

    render() {
        const {
            application,
            urls,
            files,
            handlers,
            applicationModes,
            status,
        } = this.props;
        const rows = [];

        if (application.handlerId) {
            const getHandlerVersion = (handlerId) => handlers.find(({id}) => id === handlerId).version;

            rows.push(
                <FormRow name={locale.lmsg('version')}>
                    <div className="text-value">
                        {handlers.length > 1 ? (
                            <InplaceEditor
                                value={getHandlerVersion(application.handlerId)}
                                onChange={() => {
                                    this.setState({
                                        savingField: 'handlerId'
                                    });
                                    Jsw.redirectPost(urls.changeVersion, {
                                        handlerId: application.handlerId
                                    });
                                }}
                                title={locale.lmsg('versionPopupTitle')}
                            >
                                <FormRow name={locale.lmsg('versionPopupDescription')}>
                                    <select defaultValue={application.handlerId}
                                        onChange={event => application.handlerId = event.target.value}
                                    >
                                        {handlers.map(({id, version}, key) => (
                                            <option key={key} value={id}>{version}</option>
                                        ))}
                                    </select>
                                </FormRow>
                            </InplaceEditor>
                        ) : getHandlerVersion(application.handlerId)}
                        <span> &nbsp; </span>
                        {this.state.savingField === 'handlerId' ? <span className="s-btn"><i className="icon-indicator"/> {locale.lmsg('pleaseWait')}</span> : null}
                    </div>
                </FormRow>
            );
        }

        rows.push(
            <FormRow name={locale.lmsg('documentRoot')}>
                <div className="text-value">
                    {urls.changeDocumentRoot ? (
                        <InplaceEditor
                            value={application.documentRoot}
                            onChange={() => {
                                this.setState({
                                    savingField: 'documentRoot'
                                });
                                Jsw.redirectPost(urls.changeDocumentRoot, {
                                    path: application.documentRoot
                                });
                            }}
                            title={locale.lmsg('documentRootPopupTitle')}
                        >
                            <FormRow name={locale.lmsg('documentRootPopupDescription')}>
                                <DirectoryBrowserValue
                                    rootNodeTitle={locale.lmsg('rootDirectoryTitle')}
                                    dataUrl={urls.fileManagerTreeData}
                                    onNodeClick={function (path) {
                                        if (path === '/') {
                                            this.resetSelection();
                                        } else {
                                            application.documentRoot = path;
                                        }
                                    }}
                                />
                            </FormRow>
                        </InplaceEditor>
                    ) : application.documentRoot}
                    <span> &nbsp; </span>
                    {addTrailingSlash(application.documentRoot).indexOf(addTrailingSlash(application.path)) !== 0 ? (
                        <span><i className="icon-warning" /> {locale.lmsg('documentRootNotInApplicationRootWarning')}</span>
                    ) : null}
                    <span> &nbsp; </span>
                    {this.state.savingField === 'documentRoot' ? <span className="s-btn"><i className="icon-indicator"/> {locale.lmsg('pleaseWait')}</span> : null}
                </div>
            </FormRow>
        );

        if (application.mode) {
            rows.push(
                <FormRow name={locale.lmsg('applicationMode')}>
                    <div className="text-value">
                        <InplaceEditor
                            value={application.mode}
                            onChange={() => {
                                this.setState({
                                    savingField: 'mode'
                                });
                                Jsw.redirectPost(urls.changeApplicationMode, {
                                    mode: application.mode
                                });
                            }}
                            title={locale.lmsg('applicationModePopupTitle')}
                        >
                            <FormRow name={locale.lmsg('applicationModePopupDescription')}>
                                <Dropdown value={application.mode} values={applicationModes} onChange={value => application.mode = value}/>
                            </FormRow>
                        </InplaceEditor>
                        <span> &nbsp; </span>
                        {this.state.savingField === 'mode' ? <span className="s-btn"><i className="icon-indicator"/> {locale.lmsg('pleaseWait')}</span> : null}
                    </div>
                </FormRow>
            );
        }

        if (application.url) {
            rows.push(
                <FormRow name={locale.lmsg('applicationUrl')}>
                    <div className="text-value">
                        <a target="_blank" href={application.url}>{application.url}</a>
                    </div>
                </FormRow>
            );
        }

        if (application.path) {
            rows.push(
                <FormRow name={locale.lmsg('applicationPath')}>
                    <div className="text-value">
                        {urls.changeApplicationPath ? (
                            <InplaceEditor
                                value={application.path}
                                onChange={() => {
                                    this.setState({
                                        savingField: 'path'
                                    });
                                    Jsw.redirectPost(urls.changeApplicationPath, {
                                        path: application.path
                                    });
                                }}
                                title={locale.lmsg('applicationPathPopupTitle')}
                            >
                                <FormRow name={locale.lmsg('applicationPathPopupDescription')}>
                                    <DirectoryBrowserValue
                                        rootNodeTitle={locale.lmsg('rootDirectoryTitle')}
                                        dataUrl={urls.fileManagerTreeData}
                                        onNodeClick={path => application.path = path}
                                    />
                                </FormRow>
                            </InplaceEditor>
                        ) : application.path}
                        <span> &nbsp; </span>
                        <AuxActions>
                            <a href={urls.fileManagerList + '?currentDir=' + encodeURIComponent(application.path)}>
                                {locale.lmsg('applicationPathOpen')}
                            </a>
                        </AuxActions>
                        <span> &nbsp; </span>
                        {this.state.savingField === 'path' ? <span className="s-btn"><i className="icon-indicator"/> {locale.lmsg('pleaseWait')}</span> : null}
                    </div>
                </FormRow>
            );
        }

        if (application.startupFile) {
            rows.push(
                <FormRow name={locale.lmsg('applicationStartupFile')}>
                    <div className="text-value">
                        <InplaceEditor
                            value={application.startupFile}
                            onChange={() => {
                                this.setState({
                                    savingField: 'startupFile'
                                });
                                Jsw.redirectPost(urls.changeApplicationStartupFile, {
                                    startupFile: application.startupFile
                                });
                            }}
                            title={locale.lmsg('applicationStartupFilePopupTitle')}
                        >
                            <FormRow name={locale.lmsg('applicationStartupFilePopupDescription')}>
                                <input type="text" className="form-control" defaultValue={application.startupFile}
                                    onChange={event => application.startupFile = event.target.value}
                                />
                            </FormRow>
                        </InplaceEditor>
                        <span> &nbsp; </span>
                        {files.indexOf(application.startupFile) === -1 ? (
                            <span>
                                <i className="icon-warning" /> {locale.lmsg('applicationStartupFileNotExist')}
                            </span>
                        ) : (
                            <AuxActions>
                                <a href={(startupFile => {
                                    const { currentDir, file } = splitPath(addTrailingSlash(application.path) + startupFile);
                                    return urls.fileManagerEditor
                                        + '?currentDir=' + encodeURIComponent(currentDir)
                                        + '&file=' + encodeURIComponent(file);
                                })(application.startupFile)}>{locale.lmsg('applicationStartupFileEdit')}</a>
                            </AuxActions>
                        )}
                        <span> &nbsp; </span>
                        {this.state.savingField === 'startupFile' ? <span className="s-btn"><i className="icon-indicator"/> {locale.lmsg('pleaseWait')}</span> : null}
                    </div>
                </FormRow>
            );
        }

        if (application.environment) {
            rows.push(
                <FormRow name={locale.lmsg('environment')}>
                    <div className="text-value">
                        {Object.keys(application.environment).map((name, key) => (
                            <div key={key}>- {name}: {application.environment[name]}</div>
                        ))}
                        <AuxActions>
                            <a href={urls.changeEnvironment}>{locale.lmsg('environmentEdit')}</a>
                        </AuxActions>
                    </div>
                </FormRow>
            );
        }

        let screenDescription = null;
        if (application.enabled && files.indexOf(application.startupFile) === -1) {
            screenDescription = locale.lmsg('noStartupFileHint', {
                file: (addTrailingSlash(application.path || '') + (application.startupFile || '')).escapeHTML()
            });
        }

        const operations = [];
        if (application.enabled) {
            if (urls.restart) {
                operations.push(
                    <SmallButton
                        className="sb-restart"
                        description={locale.lmsg('restartButtonDescription')}
                        onClick={() => Jsw.redirectPost(urls.restart)}
                    >{locale.lmsg('restartButton')}</SmallButton>
                );
            }
            if (files.indexOf('package.json') !== -1) {
                operations.push(
                    <SmallButton
                        className="sb-install"
                        description={locale.lmsg('npmInstallButtonDescription')}
                        onClick={() => new Ajax.Request(urls.npmInstall, {
                            method: 'post',
                            onSuccess: ({responseText}) => {
                                const data = responseText.evalJSON();
                                Jsw.getComponent('asyncProgressBarWrapper').progressDialog(data);
                            }
                        })}
                    >{locale.lmsg('npmInstallButton')}</SmallButton>
                );
                operations.push(
                    <SmallButton
                        className="sb-start"
                        description={locale.lmsg('runScriptButtonDescription')}
                        onClick={() => new RunScriptPopup({
                            url: urls.runScript
                        })}
                    >{locale.lmsg('runScriptButton')}</SmallButton>
                );
            }
            if (urls.disable) {
                operations.push(
                    <SmallButton
                        className="sb-disable"
                        description={locale.lmsg('disableButtonDescription')}
                        onClick={() => Jsw.redirectPost(urls.disable)}
                    >{locale.lmsg('disableButton')}</SmallButton>
                );
            }
        } else {
            if (urls.enable) {
                operations.push(
                    <SmallButton
                        className="sb-activate"
                        description={locale.lmsg('enableButtonDescription')}
                        onClick={() => Jsw.redirectPost(urls.enable)}
                    >{locale.lmsg('enableButton')}</SmallButton>
                );
            }
        }

        let side = null;
        if (files.length) {
            side = (
                <PanelsGroup>
                    <SidebarPanel
                        id="plesk-ext-nodejs-configuration-files-panel"
                        title={locale.lmsg('configurationFilesTitle')}
                        className="p-tools"
                    >
                        <p>{locale.lmsg('configurationFilesDescription')}</p>
                        <ul>
                            {files
                                .map(file => addTrailingSlash(application.path) + file)
                                .map(splitPath)
                                .map(({currentDir, file}, key) => (
                                    <li key={key}>
                                        <a href={urls.fileManagerEditor + '?currentDir=' + encodeURIComponent(currentDir) + '&file=' + encodeURIComponent(file)}>
                                            {file}
                                        </a>
                                    </li>
                                ))
                            }
                        </ul>
                    </SidebarPanel>
                </PanelsGroup>
            );
        }

        return (
            <Content status={status} side={side}>
                {screenDescription && <Hint expandable>{screenDescription}</Hint>}
                {operations.length ? <SmallTools>{Children.map(operations, child => cloneElement(child))}</SmallTools> : null}
                {Children.map(rows, child => cloneElement(child))}
            </Content>
        );
    };
}

export default Application;
